#!/bin/sh
# count_kernel_cap_uses.sh
#
# Copyright 2012, Michael Kerrisk, licensed under GNU GPLv2 or later
#
# Count use of all capabilities in the kernel source tree specified by $1.
# An optional second command-line argument proides a tag string that can
# is included as part of the log files produced by this script. This
# allows log files to be distinctively named for a a particular kernel
# source tree.
#
# Example usage:
#
# sh count_kernel_cap_uses.sh linux-3.2 3.2
# 
# This script produces multiple log files for each capability.
# The log files for each capability form a series (labeled p0,
# p1, p2, etc.), with each successive log file filtering out
# some of the information provided by the preceding log file.
#
# The cap_stats.sh script can be used to analyze the logs produced
# by this script
#

if test $# -lt 1; then
	echo "Usage: $0 <kernel-source-dir> [tag]" 1>&2
	exit 1
fi

tag=""
if test $# -gt 1; then
	tag="-$2"
fi

# Obtain a list of the capabilities defined in this kernel source tree

CAP_LIST=/tmp/cap_list.$$

cat $1/include/linux/capability.h | 
	grep '#def' | grep ' CAP_' | grep '[0-9]$' | 
	awk '{print $2}' | sort > $CAP_LIST

# p0: Get all usages of capability

cat $CAP_LIST | while read c; do 
	grep -r $c $1 > $c.p0.vanilla$tag.log
	echo $c
done

# p1: Lines starting "<whitespace>* " are probably comments. Discard them.

cat $CAP_LIST | while read c; do 
	cat $c.p0.vanilla$tag.log | 
		grep -v '^[ 	]*\* ' > $c.p1.nocomment$tag.log
done

# p2: Refine to just actual usages of capabilities in C source files

cat $CAP_LIST | while read c; do
	cat $c.p1.nocomment$tag.log | 
# Restrict to C source files
		grep '\.c:' | 
# All usages of capabilities are inside '()' (e.g., capable(CAP_SYS_NICE))
# (This might leave a small number of false positives, but
# they won't affect the overall results much.)
		grep '(.*CAP.*)' > $c.p2.src$tag.log; 
done

# p3: Strip out /drivers

cat $CAP_LIST | while read c; do
	cat $c.p2.src$tag.log |
		grep -v /drivers > $c.p3.nodrv$tag.log; 
done

# p4: Strip out architectures other than x86 (modern kernels) 
# or i386 (older kernels)

cat $CAP_LIST | while read c; do
	cat  $c.p3.nodrv$tag.log |
# Eliminate arch directories starting with letters other than [ix]
		grep -v '/arch/[a-hj-wyz]' |
# Filter out unwanted arch directories that may have passed
# through previous search
		grep -v '/ia64' |
		grep -v '/x86_64' > $c.p4.noarch$tag.log; 
done

# p5: Strip out file systems other than /proc

cat $CAP_LIST | while read c; do
	cat  $c.p4.noarch$tag.log |
		grep -v '/fs/[a-or-z]' > $c.p5.nofs$tag.log; 
done
